package com.cht.iot.conv;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Fetcher {
	static final Logger LOG = LoggerFactory.getLogger(Fetcher.class);
	
	final String name;
	final InputStream is;
	final OutputStream os;
	
	long writingInterval = 0L;
	
	Listener listener = new NullListener();
	
	boolean alive = true;
	
	public Fetcher(String name, InputStream is, OutputStream os) {
		this.name = name;
		this.is = is;
		this.os = os;									
	}
	
	public void setWritingInterval(long writingInterval) {
		this.writingInterval = writingInterval;
	}
	
	public void setListener(Listener listener) {
		this.listener = listener;
	}
	
	public void start() {
		Thread t = new Thread(new Runnable() {			
			public void run() {
				doFetch();					
			}
		});
		t.start();
	}
	
	public void close() throws IOException {
		alive = false;
	}
	
	public static String hex(byte[] bytes, int off, int len) {
		StringBuilder sb = new StringBuilder();
		len = len + off;
		for (int i = off;i < len;i++) {
			sb.append(String.format("%02X ", bytes[i] & 0x0FF));
		}		
		
		return sb.toString();
	}
	
	public static final void fetch(String name, InputStream is, OutputStream os, long writingInterval, Listener listener) throws IOException {
		long writingLast = 0L;
		
		byte[] bytes = new byte[255];
		int s;
		while ((s = is.read(bytes)) > 0) {
			listener.onData(bytes, 0, s);
			
			// flow control
			long now = 0;			
			while ((writingInterval > 0) &&
				   (((now = System.currentTimeMillis()) - writingLast) < writingInterval)) {
				try {
					Thread.sleep(100L);
					
				} catch (InterruptedException e) {					
				}
			}
			
			os.write(bytes, 0, s);
				
			writingLast = now;
			
			LOG.info("{} - {}", name, hex(bytes, 0, s));
		}
	}
	
	void doFetch() {
		while (alive) {
			try {			
				fetch(name, is, os, writingInterval, listener);
				
			} catch (Exception e) {
				LOG.error(e.getMessage(), e);
			}
		}
		
		LOG.info("Fetcher is closed");
	}
	
	public static interface Listener {
		void onData(byte[] bytes, int offset, int length);
	}
	
	static class NullListener implements Listener {
		
		public void onData(byte[] bytes, int offset, int length) {
		}
	}
}
